﻿using System;
using System.Windows.Forms;

///////////////////////////////////////////////////////////////////////////////////////////////////
// RootPro CADメインウィンドウのWindowsイベントをフックするNativeWindowクラス
// ActiveTabView が取得できるまでマウスイベントを監視し、取得できたらActiveTabViewの
// ウィンドウハンドルをセットする
// Copyright @ 2019 RootPro Co.,Ltd. 
///////////////////////////////////////////////////////////////////////////////////////////////////
namespace RCAddInAdvancedZoomCS
{
    class RCMainNativeWindow : NativeWindow
    {
        private const int WM_SETCURSOR = 0x20;

        AppAddIn app;
        IntPtr prevTabViewWindowHandle = (IntPtr)(-1);

        public RCMainNativeWindow(AppAddIn app)
        {
            this.app = app;
        }

        protected override void WndProc(ref System.Windows.Forms.Message m)
        {
            // ActiveTabView のウィンドウハンドルが取得できるまで
            // RootPro CAD メインウィンドウのマウスイベントを監視する。
            if (m.Msg >= WM_SETCURSOR)
            {
                // ActiveTabView が取れるまでは何もしない
                if (app.ActiveTabView != null)
                {
                    //ひとつ前のWindowHandle と変化ない場合は変わるまで何もしない
                    if (app.ActiveTabView.WindowHandle != prevTabViewWindowHandle)
                    {
                        // アクティブになった新しいTabVeiw のウィンドウハンドルをセット
                        prevTabViewWindowHandle = app.ActiveTabView.WindowHandle;
                        app.SetWindowHandle();
                    }
                }
            }
            base.WndProc(ref m);
        }
    }
}
