﻿using System;
using System.Threading;
using System.Reflection;
using System.IO;
using RootPro.RootProCAD;

namespace RCAddInAdvancedZoomCS
{
    [System.AddIn.AddIn("AppAddIn", Version = "1.0", Publisher = "", Description = "")]
    public partial class AppAddIn
    {
        // タブビュー用NativeWindow
        private RCTabViewNativeWindow pnwTabView;
        // RootPro CAD メインウィンドウ用NativeWindow
        private RCMainNativeWindow pnwRPMain;

        private void AppAddIn_Startup(object sender, EventArgs e)
        {
            AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(ResolveAssembly);

            // タブビュー用NativeWindow
            pnwTabView = new RCTabViewNativeWindow(this);

            // RootPro CAD メインウィンドウ用NativeWindow
            pnwRPMain = new RCMainNativeWindow(this);

            //アクティブな図面ファイルが変更されたときに呼ばれるイベントハンドラを追加
            this.DocumentActivated += new DocumentEventHandler(app_DocumentActivated);

            //部分図が変更されたときに呼ばれるイベントハンドラを追加
            this.CurrentDrawingChanged += new CurrentDrawingChangedEventHandler(app_CurrentDrawingChanged);

            //「アドインの管理」画面からアドインを読み込んだ場合は、ActiveTabView が取得できるので
            // ActiveTabView のウィンドウハンドルを登録する
            if (ActiveTabView != null)
            {
                //ActiveTabView のウィンドウハンドルを登録
                pnwTabView.AssignHandle(ActiveTabView.WindowHandle);
            }
            // RootProCAD起動時に本アドインが読み込まれる場合は、Startupの時点でActiveTabViewが
            // 取得できないため、ActiveTabView が見つかるまでメインウィンドウで監視する
            else
            {
                //RootPro CAD のメインウィンドウハンドル
                pnwRPMain.AssignHandle(this.WindowHandle);
            }
        }

        private void AppAddIn_Shutdown(object sender, EventArgs e)
        {
            //イベントハンドラを開放
            this.CurrentDrawingChanged -= new CurrentDrawingChangedEventHandler(app_CurrentDrawingChanged);
            this.DocumentActivated -= new DocumentEventHandler(app_DocumentActivated);
        }

        #region VSTA generated code
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(AppAddIn_Startup);
            this.Shutdown += new System.EventHandler(AppAddIn_Shutdown);
        }
        #endregion

        /// <summary>
        /// カレント部分図が変更されたときに呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void app_CurrentDrawingChanged(object sender, CurrentDrawingChangedEventArgs e)
        {
            //古いウィンドウハンドルを開放
            if (pnwRPMain.Handle != (IntPtr)0)
                pnwRPMain.ReleaseHandle();

            //TabView が変更された場合はActiveTabView のウィンドウハンドルを設定
            if (ActiveTabView != null)
            {
                if (pnwTabView.Handle != (IntPtr)0)
                    pnwTabView.ReleaseHandle();
                //ActiveTabView のウィンドウハンドルを登録
                pnwTabView.AssignHandle(ActiveTabView.WindowHandle);

                pnwTabView.ResetLastViewPoint();
            }
            //TabView が閉じられた場合はActiveTabView が見つかるまでメインウィンドウで監視
            else
            {
                //RootPro CAD のメインウィンドウハンドル
                pnwRPMain.AssignHandle(this.WindowHandle);
            }
        }


        /// <summary>
        /// アクティブな図面ファイルが変更されたときに呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void app_DocumentActivated(object sender, DocumentEventArgs e)
        {
            //古いウィンドウハンドルを開放
            if (pnwRPMain.Handle != (IntPtr)0)
                pnwRPMain.ReleaseHandle();

            if (e.Document != null)
            {
                //RootPro CAD のメインウィンドウで監視
                pnwRPMain.AssignHandle(this.WindowHandle);
            }

        }

        /// <summary>
        /// ActiveTabView のウィンドウハンドルを登録する
        /// </summary>
        public void SetWindowHandle()
        {
            if (pnwTabView.Handle != (IntPtr)0)
                pnwTabView.ReleaseHandle();
            pnwTabView.AssignHandle(ActiveTabView.WindowHandle);

            pnwTabView.ResetLastViewPoint();
        }

        private Assembly ResolveAssembly(object sender, ResolveEventArgs args)
        {
            AssemblyName assemblyName = new AssemblyName(args.Name);

            foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                if (AssemblyName.ReferenceMatchesDefinition(assemblyName, assembly.GetName()))
                    return assembly;
            }

            if (assemblyName.Name.EndsWith(".resources"))
            {
                if (assemblyName.CultureName.Equals(Thread.CurrentThread.CurrentUICulture.Name, StringComparison.OrdinalIgnoreCase))
                {
                    string dllPath = Path.GetDirectoryName(this.AddInFullPath);
                    dllPath = Path.Combine(dllPath, assemblyName.CultureName);
                    dllPath = Path.Combine(dllPath, assemblyName.Name + ".dll");

                    if (File.Exists(dllPath))
                    {
                        try
                        {
                            return Assembly.LoadFrom(dllPath);
                        }
                        catch
                        {
                        }
                    }
                }
            }

            return null;
        }

    }
}
